import 'dart:async';
import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:viqcore/viqcore.dart';

/// LocaleObject.
/// 自定义本地化支持.
class LocaleObject {
  LocaleObject(this.locale){
    current = this;
  }

  /// Only use if no access to context
  static LocaleObject current;

  Locale locale;

  /// get string by id,Can be specified languageCode,countryCode.
  /// 通过id获取字符串,可指定languageCode,countryCode.
  String getString(String id,
      {String languageCode, String countryCode, List<Object> params}) {
    if (!_localizedValues.keys.contains(id)) {
      return 'Language id "$id" is not found';
    }

    final _languageCode = languageCode ?? locale.languageCode;
    final _countryCode = countryCode ?? locale.countryCode;
    final _languageCountryCode =
        _countryCode == null ? _languageCode : '$_languageCode-$_countryCode';

    Map<String, dynamic> idMap = _localizedValues[id];

    String value =
        idMap[_languageCountryCode] ?? idMap[_languageCode] ?? idMap[0];

    if (params != null && params.isNotEmpty) {
      for (int i = 0, length = params.length; i < length; i++) {
        value = value?.replaceAll('{$i}', '${params[i]}');
      }
    }
    return value;
  }

  static const String fieldDescription = '__description__';
  static const String _storedLocaleId = '_LocaleObjectId_';

  static List<LocaleModel> localeModels() {
    Map<String, String> supportedLanguageMap = _localizedValues[LangSysIds.supportedLanguage] ?? {};
    List<LocaleModel> list = supportedLanguageMap.keys
        .map((code) => LocaleModel(code, supportedLanguageMap[code]))
        .toList();
    return list;
  }

  static Future<bool> init() async {
    await _loadLanguage();
    //await _loadCustom();
    await _loadStoredLocale();
    return Future.value(true);
  }

  static LocaleObject of(BuildContext context) {
    return Localizations.of<LocaleObject>(context, LocaleObject);
  }

  static Future _loadStoredLocale() async {
    _selectedLocaleCode = await Prefs.getStringF(_storedLocaleId, "");
    _selectedLocale = null;
    if (_selectedLocaleCode.isNotEmpty) {
      _selectedLocale = _getLocale(_selectedLocaleCode);
    }
  }

  static Future _storeLocale(String code) async {
    await Prefs.setString(_storedLocaleId, code);
    await _loadStoredLocale();
    App.refresh();
  }

  static void showLanguagePicker(BuildContext context) {
    LanguagePicker.showLanguagePicker(context: context);
  }

  static Future _loadLanguage() async {
    try {
      String path = "assets/lang";
      String systemData = await rootBundle.loadString('$path/_.json');
      Map<String, dynamic> systemMap = json.decode(systemData);
      String customData = await rootBundle.loadString('$path/custom.json');
      Map<String, dynamic> customMap = json.decode(customData);

      systemMap.forEach((id, idMap) {
        Map<String, String> langMap = {};
        idMap.forEach((lang, translation) {
          langMap[lang] = translation;
        });
        _localizedValues[id] = langMap;
      });

      customMap.forEach((id, idMap) {
        Map<String, String> langMap = {};
        idMap.forEach((lang, translation) {
          langMap[lang] = translation;
        });
        _localizedValues[id] = langMap;
      });
    } catch (e) {
      print('Exception in LocaleObject._loadLanguage: $e');
    }
  }

  /// supported Locales
  /// 支持的语言
  static Iterable<Locale> supportedLocales = _getSupportedLocales();
  static Map<String, Map<String, String>> _localizedValues = {};
  static Locale get selectedLocale => _selectedLocale;
  static Locale _selectedLocale;
  static String get selectedLocaleCode => _selectedLocaleCode;
  static String _selectedLocaleCode;

  static List<Locale> _getSupportedLocales() {
     Map<String, String> supportedLanguageMap =
        _localizedValues[LangSysIds.supportedLanguage] ?? {};
    return supportedLanguageMap.keys.map((code) => _getLocale(code)).toList();
  }

  static Locale _getLocale(String code) {
    if (code.contains('-')) {
      var token = code.split('-');
      return Locale(token[0], token[1]);
    }
    return Locale(code);
  }

  static bool _isSupported(Locale locale) {
    Map<String, String> supportedLanguageMap =
        _localizedValues[LangSysIds.supportedLanguage] ?? {};
    return supportedLanguageMap.keys.contains(locale.languageCode) ||
        (locale.countryCode != null &&
            _localizedValues.keys
                .contains('${locale.languageCode}-${locale.countryCode}')) ||
        supportedLanguageMap.keys
            .any((key) => key.contains(locale.languageCode));
  }

  static const LocalizationsDelegate<LocaleObject> delegate =
      _LocaleObjectDelegate();
}

class _LocaleObjectDelegate extends LocalizationsDelegate<LocaleObject> {
  const _LocaleObjectDelegate();

  @override
  bool isSupported(Locale locale) => LocaleObject._isSupported(locale);

  @override
  Future<LocaleObject> load(Locale locale) {
    return new SynchronousFuture<LocaleObject>(new LocaleObject(locale));
  }

  @override
  bool shouldReload(_LocaleObjectDelegate old) => false;
}

class LocaleModel {
  final String code;
  final String description;
  bool get isSelected => code == LocaleObject._selectedLocaleCode;
  LocaleModel(this.code, this.description);

  void updateSelected() {
    LocaleObject._storeLocale(code);
  }

  static LocaleModel autoLanguage(BuildContext context) {
    return LocaleModel(
        '', LocaleObject.of(context).getString(LangSysIds.languageAuto));
  }
}
