import 'dart:io';

import 'package:flutter/material.dart';
import 'package:validators/validators.dart' as validators;
import 'package:viqcore/viqcore.dart';

///CircularProfileAvatar allows developers to implement circular profile avatar with border,
/// overlay, initialsText and many other features which simplifies developer's job.
/// It is an alternative to Flutter's CircleAvatar Widget.
class CircularProfileAvatar extends StatefulWidget {
  CircularProfileAvatar(this.imageString,
      {Key key,
      this.initialsText = const Text(''),
      this.radius = 50.0,
      this.borderWidth = 0.0,
      this.borderColor = Colors.white,
      this.backgroundColor = Colors.white,
      this.elevation = 0.0,
      this.showInitialTextAbovePicture = false,
      this.onTap,
      this.foregroundColor = Colors.transparent})
      : super(key: key);

  /// sets radius of the avatar circle, [borderWidth] is also included in this radius.
  /// default value is 0.0
  final double radius;

  /// sets shadow of the circle,
  /// default value is 0.0
  final double elevation;

  /// sets the borderWidth of the circile,
  /// default value is 0.0
  final double borderWidth;

  /// The color with which to fill the border of the circle.
  /// default value [Colors.white]
  final Color borderColor;

  /// The color with which to fill the circle.
  /// default value [Colors.white]
  final Color backgroundColor;

  /// sets the [foregroundColor] of the circle, It only works if [showInitialTextAbovePicture] is set to true.
  /// [foregroundColor] doesn't include border of the circle.
  final Color foregroundColor;

  /// it takes a URL of the profile image.
  final String imageString;

  /// Sets the initials of user's name.
  final Text initialsText;

  /// Displays initials above profile picture if set to true, You can set [foregroundColor] value as well if [showInitialTextAbovePicture]
  /// is set to true.
  final bool showInitialTextAbovePicture;

  /// sets onTap gesture.
  final GestureTapCallback onTap;

  @override
  _CircularProfileAvatarState createState() => _CircularProfileAvatarState();
}

class _CircularProfileAvatarState extends State<CircularProfileAvatar> {
  Widget _initialsText;

  @override
  Widget build(BuildContext context) {
    _initialsText = Center(child: widget.initialsText);

    return GestureDetector(
      onTap: widget.onTap,
      child: Material(
        type: MaterialType.circle,
        elevation: widget.elevation,
        color: widget.borderColor,
        child: Container(
            height: widget.radius * 2,
            width: widget.radius * 2,
            padding: EdgeInsets.all(widget.borderWidth),
            decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(widget.radius),
                color: widget.borderColor),
            child: Center(
              child: Container(
                decoration: BoxDecoration(
                    color: widget.backgroundColor,
                    borderRadius: BorderRadius.circular(widget.radius)),
                child: Stack(
                  fit: StackFit.expand,
                  children: widget.showInitialTextAbovePicture
                      ? <Widget>[
                          profileImage(),
                          Container(
                            decoration: BoxDecoration(
                                color: widget.foregroundColor,
                                borderRadius:
                                    BorderRadius.circular(widget.radius)),
                          ),
                          _initialsText,
                        ]
                      : <Widget>[_initialsText, profileImage()],
                ),
              ),
            )),
      ),
    );
  }

  Widget profileImage() {
    if (widget.imageString == null) {
      return CircleAvatar(
        backgroundColor: Colors.grey,
        child: Icon(Icons.person, size: widget.radius),
      );
    }

    if (validators.isURL(widget.imageString)) {
      return CircleAvatar(
        backgroundImage:
            NetworkImage(WebClient.current.appendBasicAuth(widget.imageString)),
      );
    }

    return ClipRRect(
      borderRadius: BorderRadius.circular(widget.radius),
      child: Image.file(
        File.fromUri(
          Uri(
            path: widget.imageString,
          ),
        ),
      ),
    );
  }
}
