import 'package:flutter/material.dart';
import 'package:viqcore/viqcore.dart';

class DataListController extends DataController {
  static const String fieldTotalCount = 'totalCount';

  List<DataObject> list = [];

  DataListFormController get dataListFormController => _dataListFormController;
  DataListFormController _dataListFormController;

  @protected
  int pagingLoadIndex = -1;
  int get nextPagingIndex => pagingLoadIndex + 1;
  bool get isPagingLoad => pagingLoadIndex >= 0;
  void movePagingIndex(WebClientResult result, [int fetchCount = -1]) {
    if (!result.ok) return;

    if (pagingLoadIndex == -1 &&
        fetchCount > -1 &&
        result.data != null &&
        result.data[fieldTotalCount] != null) {
      _isEndOfList = result.data[fieldTotalCount] < fetchCount;
    }
    pagingLoadIndex++;
  }

  void resetPagingLoad() {
    pagingLoadIndex = -1;
    _isEndOfList = false;
  }

  bool get isEndOfList => _isEndOfList;
  bool _isEndOfList = false;

  Map<String, String> sortingFields; // <fieldName, label>
  final DataObjectBuilder dataObjectBuilder;

  ///[dataObjectBuilder]: () => CustomDataObject()
  DataListController({@required this.dataObjectBuilder, this.sortingFields});

  @override
  void dispose() {
    list = null;
    super.dispose();
  }

  Future<DataResponse> rebuild() {
    clear();
    return reload();
  }

  Future<DataResponse> reload() {
    resetPagingLoad();
    return asyncInProgress(load(rebuild: true),
        title: translation(stateMVC.context, LangSysIds.loading));
  }

  Future<DataResponse> load({bool rebuild = false}) async {
    if (isEndOfList) return null;
    WebClientResult result = await onLoad();
    if (result.ok) {
      int beforeLoaded = list.length;
      if (rebuild) {
        clear();
      }
      List<dynamic> dataList = result.data;
      dataList.forEach((data) {
        var dataObject = dataObjectBuilder();
        dataObject.fill(data);
        list.add(dataObject);
      });
      if (this.isPagingLoad && !_isEndOfList) {
        _isEndOfList = beforeLoaded == list.length;
      }
      onListLoaded();
    }
    return DataResponse.fromWebClientResult(result);
  }

  void delete(int index) {
    DataObject dataObject = list[index];
    asyncInProgress(dataObject.delete()
      ..then((response) {
        if (response.ok) list.removeAt(index);
      }));
  }

  void add(DataObject dataObject, {bool addOnTop = false}) {
    if (addOnTop)
      list.insert(0, dataObject);
    else
      list.add(dataObject);
  }

  void sort([int compare(DataObject a, DataObject b)]) {
    list.sort(compare);
    this.refresh();
  }

  /// Sorts by the field specified in the required parameter fieldName.
  /// Optionally specify sortOrder to sort in ascending or descending order.
  /// Defaults to ascending order.
  void sortByField(String fieldName,
      {SortOrder sortOrder: SortOrder.ascending}) {
    if (sortOrder == SortOrder.ascending) {
      sort((a, b) {
        //handle null fields
        if (a[fieldName] == null && b[fieldName] == null) return 0;
        if (a[fieldName] == null && b[fieldName] != null) return -1;
        if (a[fieldName] != null && b[fieldName] == null) return 1;
        return a[fieldName].compareTo(b[fieldName]);
      });
    } else {
      sort((a, b) {
        if (a[fieldName] == null && b[fieldName] == null) return 0;
        if (a[fieldName] == null && b[fieldName] != null) return 1;
        if (a[fieldName] != null && b[fieldName] == null) return -1;
        return b[fieldName].compareTo(a[fieldName]);
      });
    }
  }

  Future<WebClientResult> onLoad() async {
    throw ('Not implemented');
  }

  void clear(){
    list.clear();
    onListClear();
  }

  void onListLoaded() {}
  void onListClear() {}

  static of(BuildContext context, {bool listen = false}) {
    return Provider.of<DataListController>(context, listen: listen);
  }

  @mustCallSuper
  void initForm() {}

  @mustCallSuper
  void disposeForm() {
    _dataListFormController = null;
  }

  bool onFormBeforeSave() {
    return true;
  }

  void onFormAfterSaveSuccess() {}

  /// If [index] is not provided, it could be mean for adding new dataObject.
  void openEditForm(BuildContext context, WidgetBuilder dataFormBuilder,
      {int index = -1}) {
    DataObject dataObject = index > -1 ? list[index] : dataObjectBuilder();
    _dataListFormController = DataListFormController(
        dataListController: this, dataObject: dataObject);
    _dataListFormController.open(context, dataFormBuilder);
  }

  void openViewForm(BuildContext context, WidgetBuilder dataFormBuilder,
      {@required int index}) {
    _dataListFormController = DataListFormController(
        dataListController: this, dataObject: list[index], isViewOnly: true);
    _dataListFormController.open(context, dataFormBuilder);
  }

  void closeForm() {
    _dataListFormController?.close();
  }
}

class DataListFormController extends DataFormController with FlutterFormMixin {
  final DataListController dataListController;
  final bool addNewOnTop;

  DataListFormController({
    @required this.dataListController,
    DataObject dataObject,
    this.addNewOnTop = true,
    bool isViewOnly = false,
  }) : super(
            title: dataListController.title,
            dataObject: dataObject,
            isViewOnly: isViewOnly) {
    dataListController.initForm();
  }

  @override
  String get title => dataListController.title;

  static of(BuildContext context, {bool listen = false}) {
    return Provider.of<DataFormController>(context, listen: listen)
        as DataListFormController;
  }

  void open(BuildContext context, WidgetBuilder dataFormBuilder) {
    Navigator.push(context, MaterialPageRoute(
      builder: (BuildContext context) {
        return DataFormScaffold(
          dataFormController: this,
          dataFormBuilder: dataFormBuilder,
        );
      },
    ));
  }

  void close() {
    Navigator.pop(stateMVC.context);
  }

  @override
  void dispose() {
    dataListController.disposeForm();
    super.dispose();
  }

  @override
  void onSaveSuccess(bool isNew) {
    if (isNew) {
      dataListController.add(dataObject, addOnTop: addNewOnTop);
    }
    dataListController.refresh();
    close();
  }
}

enum SortOrder { ascending, descending }
