import 'package:flutter/material.dart';

import 'package:viqcore/viqcore.dart';

/// A ListView that automatically displays the contents of a DataList.
/// By default it includes an action button that allows deletion and
/// editing of each document. To replace the default ListTile,
/// supply a listItemBuilder. The look and feel of the ListView
/// can be customized by providing a decoration. emptyListWidget, if supplied,
/// will display in the case that the DataList is empty.
class DataListView extends StatefulWidget {
  /// A call back function for building custom widgets for the DataListView,
  /// rather than the default ListTile. Like a normal builder, it receives a
  /// integer as an index into the associated DataList and should return a
  /// widget.
  /// Widget listItemBuilder(int index) {/* create and return a custom widget
  /// for documentList[index]*/}
  final IndexedWidgetBuilder listItemBuilder;

  final IndexedWidgetBuilder listHeaderBuilder;

  /// A widget to display when the DataListView is empty (when the
  /// DataList.length == 0)
  final Widget emptyListWidget;

  /// Control the scroll axis. Defaults to Vertical. Ignored if
  /// listItemBuilder is null.
  final Axis scrollDirection;

  /// A BoxDecoration for the whole list view
  final BoxDecoration decoration;

  final WidgetBuilder dataFormBuilder;

  DataListView(
      {@required this.listItemBuilder,
      this.dataFormBuilder,
      this.listHeaderBuilder,
      this.emptyListWidget,
      this.decoration,
      this.scrollDirection: Axis.vertical});

  _DataListViewState createState() => _DataListViewState();
}

class _DataListViewState extends StateMVC<DataListView> {
  DataListController dataListController;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    if (dataListController == null) {
      //dataListController = ScopedModelObject.of<DataListController>(context);
      dataListController = DataListController.of(context);
      this.add(dataListController);
      Future.delayed(Duration.zero, () => dataListController.rebuild());
    }
  }

  @override
  Widget build(BuildContext context) {
    if (widget.emptyListWidget != null &&
        !dataListController.inAsyncCall &&
        dataListController.list.length == 0) {
      return widget.emptyListWidget;
    }

    return Container(
      decoration: widget.decoration,
      child: RefreshIndicator(
        onRefresh: () async {
          dataListController.reload();
        },
        child: widget.listHeaderBuilder == null
            ? _listViewBuilder(context)
            : _listViewSeparatedByGroupHeader(context),
      ),
    );
  }

  Widget _listViewBuilder(BuildContext context) {
    return ListView.builder(
        scrollDirection: widget.scrollDirection,
        itemCount: dataListController.list.length +
            (dataListController.isPagingLoad ? 1 : 0),
        itemBuilder: (context, index) {
          if (index < dataListController.list.length) {
            return widget.listItemBuilder(context, index);
          } else if (index == dataListController.list.length) {
            if (dataListController.isEndOfList) {
              return Center(
                  child: Text(translation(context, LangSysIds.endOfRecords)));
            }
            dataListController.load().whenComplete(() => this.refresh());
            return defaultLoading();
          }
        });
  }

  Widget _listViewSeparatedByGroupHeader(BuildContext context) {

    return ListView.separated(
        scrollDirection: widget.scrollDirection,
        itemCount: dataListController.list.length +
            (dataListController.isPagingLoad ? 1 : 0),
        itemBuilder: (context, index){
            if ( index >= dataListController.list.length) return Container();
            return widget.listHeaderBuilder(context, index);
        },
        separatorBuilder: (context, index) {
          if (index < dataListController.list.length) {
            return widget.listItemBuilder(context, index);
          } else if (index == dataListController.list.length) {
            if (dataListController.isEndOfList) {
              return Center(
                  child: Text(translation(context, LangSysIds.endOfRecords)));
            }
            dataListController.load().whenComplete(() => this.refresh());
            return defaultLoading();
          }
        });
  }

  Widget defaultLoading() {
    return Align(
      child: SizedBox(
        height: 40,
        width: 40,
        child: Padding(
          padding: const EdgeInsets.all(8),
          child: CircularProgressIndicator(),
        ),
      ),
    );
  }
}
