import 'package:flutter/material.dart';
import 'package:meta/meta.dart';
import 'package:viqcore/viqcore.dart';

typedef DataObjectBuilder = DataObject Function();

abstract class DataObject {
  Map<String, dynamic> get map => _map;
  final Map<String, dynamic> _map = Map<String, dynamic>();

  dynamic operator [](String fieldName) => _map[fieldName];

  void operator []=(String fieldName, dynamic value) {
    map[fieldName] = value;
    if (valueChangedCallback != null &&
        valueChangedCallback[fieldName] != null) {
      valueChangedCallback[fieldName](value);
    }
  }

  Map<String, DataFieldFormatter> formatters;
  Map<String, ValueChanged<dynamic>> valueChangedCallback;

  bool _isRaw = false;
  bool get isNew => !_isRaw;

  DataObject({Map<String, dynamic> map}) {
    map?.keys?.forEach((fieldName) => this[fieldName] = map[fieldName]);
  }

  @mustCallSuper
  void initState() {}

  @mustCallSuper
  Future<bool> init() async {
    return Future.value(true);
  }

  Future<DataResponse> load(dynamic id) async {
    WebClientResult result = await onLoad(id);
    if (result.ok) {
      fill(result.data);
    }
    return DataResponse.fromWebClientResult(result);
  }

  Future<DataResponse> delete() async {
    WebClientResult result = await onDelete();
    return DataResponse.fromWebClientResult(result);
  }

  Future<DataResponse> update() async {
    WebClientResult result = await onUpdate();
    if (result.ok) {
      fill(result.data);
    }
    return DataResponse.fromWebClientResult(result);
  }

  Future<DataResponse> create() async {
    WebClientResult result = await onCreate();
    if (result.ok) {
      fill(result.data);
    }
    return DataResponse.fromWebClientResult(result);
  }

  Future<DataResponse> save() async {
    return isNew ? await create() : await update();
  }

  void updateValues(Map<String, dynamic> data) {
    data.forEach((fieldName, value) {
      if (value != null) {
        this[fieldName] = value;
      }
    });
  }

  @mustCallSuper
  void fill(Map<String, dynamic> data) {
    _isRaw = true;
    data.forEach((fieldName, value) {
      if (value != null) {
        DataFieldFormatter formatter =
            formatters == null ? null : formatters[fieldName];
        this[fieldName] = formatter == null ? value : formatter(value);
      }
    });
  }

  @protected
  void clear() {
    map.clear();
  }

  @mustCallSuper
  void dispose() {
    map.clear();
  }

  @protected
  Future<WebClientResult> onLoad(dynamic id) async {
    throw ('Not implemented');
  }

  @protected
  Future<WebClientResult> onDelete() async {
    throw ('Not implemented');
  }

  @protected
  Future<WebClientResult> onUpdate() async {
    throw ('Not implemented');
  }

  @protected
  Future<WebClientResult> onCreate() async {
    throw ('Not implemented');
  }

  dynamic get<T>(String key, {T defaultValue}) {
    if (_map.containsKey(key)) {
      if (T != null && _map[key] is T) {
        final T data = _map[key];
        return data;
      } else {
        return _map[key];
      }
    } else {
      return defaultValue;
    }
  }
}

// abstract class BaseObject {
//   /// Stores all the values of a class
//   //@protected
//   Map<String, dynamic> _map = Map<String, dynamic>();

//   void clear() {
//     _map.clear();
//   }
//   // dynamic operator [](String key) => map[key];

//   // void operator []=(String key, dynamic value) {
//   //   map[key] = value;
//   // }

//   // /// Sets type [T] from objectData
//   // ///
//   // /// To set an int, call setType<int> and an int will be saved
//   // /// [bool] forceUpdate is always true, if unsure as to whether an item is
//   // /// needed or not, set to false
//   // void set<T>(String key, T value, {bool forceUpdate = true}) {
//   //   if (value != null) {
//   //     if (_map.containsKey(key)) {
//   //       if (forceUpdate) {
//   //         _map[key] = value;
//   //       }
//   //     } else {
//   //       _map[key] = value;
//   //     }
//   //   }
//   // }

//   /// Gets type [T] from objectData
//   ///
//   /// Returns null or [defaultValue] if provided. To get an int, call
//   /// getType<int> and an int will be returned, null, or a defaultValue if
//   /// provided
//   dynamic get<T>(String key, {T defaultValue}) {
//     if (_map.containsKey(key)) {
//       if (T != null && _map[key] is T) {
//         final T data = _map[key];
//         return data;
//       } else {
//         return _map[key];
//       }
//     } else {
//       return defaultValue;
//     }
//   }

//   @mustCallSuper
//   void dispose() {}
// }

class DataResponse {
  bool ok;
  String error;
  String errorDetails;

  DataResponse({this.ok, this.error, this.errorDetails});

  static DataResponse failed(String error) {
    return DataResponse(ok: false, error: error);
  }

  static DataResponse success() {
    return DataResponse(ok: true);
  }

  static fromWebClientResult(WebClientResult result, {String errorText}) {
    var error;
    var errorDetails;
    if (!result.ok) {
      if (errorText == null) {
        error = result.error;
      } else {
        error = errorText;
        errorDetails = result.error;
      }
    }
    return DataResponse(
        ok: result.ok, error: error, errorDetails: errorDetails);
  }

  void notifyError(GlobalKey<ScaffoldState> scaffoldKey, {SnackBar snackbar}) {
    if (this.ok || scaffoldKey == null) return;
    scaffoldKey?.currentState?.showSnackBar(snackbar);
  }
}
