import 'dart:convert';
import 'dart:math';
import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:time_machine/time_machine.dart';
import 'package:viqcore/viqcore.dart';

abstract class DataUtils {
  static Future saveLocal(String id, Map<String, dynamic> map) async {
    String mapString = json.encode(map);
    await Prefs.setString(id, mapString);
  }

  static Future<Map<String, dynamic>> loadLocal(String id) async {
    String mapString = await Prefs.getStringF(id, '{}');
    try {
      return json.decode(mapString);
    } catch (e) {
      return Map<String, dynamic>();
    }
  }

  static dynamic getMapValue(Map<String, dynamic> map, String key,
      {dynamic defaultValue}) {
    if (map?.containsKey(key) ?? false) {
      return map[key];
    } else {
      return defaultValue;
    }
  }

  ///  * Examples of accepted strings:
  ///  *
  ///  * * `"2012-02-27 13:27:00"`
  ///  * * `"2012-02-27 13:27:00.123456z"`
  ///  * * `"2012-02-27 13:27:00,123456z"`
  ///  * * `"20120227 13:27:00"`
  ///  * * `"20120227T132700"`
  ///  * * `"20120227"`
  ///  * * `"+20120227"`
  ///  * * `"2012-02-27T14Z"`
  ///  * * `"2012-02-27T14+00:00"`
  ///  * * `"-123450101 00:00:00 Z"`: in the year -12345.
  ///  * * `"2002-02-27T14:00:00-0500"`: Same as `"2002-02-27T19:00:00Z"`
  static DateTime toLocalDateTime(String formattedString) {
    return Instant.dateTime(DateTime.parse(formattedString))
        .inLocalZone()
        .toDateTimeLocal();
  }

  /// This attempts to intelligently cast any value to the desired type.
  static T intelligentCast<T>(dynamic value) {
    // instead of empty string we want to return null
    if (value.toString().isEmpty) return null;

    // already expected type, return it
    if (value is T) return value;

    // process bool here, since we need to return false rather than null
    if (T.toString() == 'bool') return boolParse(value.toString()) as T;

    // if null, dont change it.
    if (value == null) return null;

    // perform explicit parsing on the value as a string
    if (T.toString() == 'double') return double.parse(value.toString()) as T;
    if (T.toString() == 'int') return int.parse(value.toString()) as T;
    if (T.toString() == 'Color') return colorParse(value) as T;

    throw Exception('Failed to convert $value to $T');
  }

  /// Given a DateTime this will replace just the date portion leaving the time unchanged
  static DateTime updateJustDate(DateTime newDate, DateTime originalDateTime) {
    return DateTime(
      newDate.year, // year
      newDate.month, // month
      newDate.day, // day
      originalDateTime.hour, // hour
      originalDateTime.minute, // minute
      originalDateTime.second, // second
      originalDateTime.millisecond, // millisecond
      originalDateTime.microsecond, // microsecond
    );
  }

  /// Given a DateTime this will replace just the time portion leaving the date unchanged
  static DateTime updateJustTime(TimeOfDay newTime, DateTime originalDateTime) {
    return DateTime(
      originalDateTime.year, // year
      originalDateTime.month, // month
      originalDateTime.day, // day
      newTime.hour, // hour
      newTime.minute, // minute
      0, // second
      0, // millisecond
      0, // microsecond
    );
  }

  /// This will parse various conceptual representaitons of yes/no into a boolean
  static bool boolParse(String value) {
    if (value == null) return false;

    if (value.toLowerCase() == 'true') return true;
    if (value.toLowerCase() == 'false') return false;

    if (value.toLowerCase() == '1') return true;
    if (value.toLowerCase() == '0') return false;

    if (value.toLowerCase() == 'yes') return true;
    if (value.toLowerCase() == 'no') return false;

    return false;
  }

  /// This will parse either a string or integer representation of a color into an actual Color
  static Color colorParse(dynamic value) {
    // input is string (e.g. 'FF112233') convert to integer
    if (value is String) {
      if (value.toString().length == 6) value = 'FF' + value.toString();
      value = int.parse(value.toString(), radix: 16);
    }

    // input is integer (e.g. 0xFF112233) convert to color
    if (value is int) {
      return Color(value).withOpacity(1.0);
    }

    throw Exception('Failed to convert $value to Color');
  }

  /// This will convert a Color to a hex string (more abreviated than the .toString() method.
  static String colorToString(Color color) {
    return color.toString().split('(0x')[1].split(')')[0];
  }

  /// used to reverse the value from an mask_formatter controller
  static String unmaskValue(String mask, String maskedValue) {
    String specialCharacters = '0A@*';
    String unmaskedValue = '';

    for (int i = 0; i < min(mask.length, maskedValue.length); i++) {
      if (specialCharacters.contains(mask[i])) unmaskedValue += maskedValue[i];
    }

    return unmaskedValue;
  }

  static DateTime get today => getStartOfDay(DateTime.now());

  static DateTime get yesterday => today.subtract(Duration(days: 1));

  static DateTime get startOfWeek => getStartOfWeek(DateTime.now());

  static DateTime getStartOfDay(DateTime originalDateTime) {
    return DateTime(
      originalDateTime.year, // year
      originalDateTime.month, // month
      originalDateTime.day, // day
      0, // hour
      0, // minute
      0, // second
      0, // millisecond
      0, // microsecond
    );
  }

   static DateTime getStartOfWeek(DateTime originalDateTime) {
    DateTime sod = getStartOfDay(originalDateTime);
    return sod.subtract(Duration(days: sod.weekday));
  }
}
