import 'dart:async';
import 'dart:core';
import 'dart:convert';
import 'package:http/http.dart' as http;

typedef WebClientStringCallback = String Function();

class WebClient {
  final WebClientStringCallback
      baseUrlCallback; // = 'http://104.248.147.117:5011';
  final WebClientStringCallback basicAuthCallback;
  final WebClientStringCallback jwtTokenCallback;
  //final WebClientSetToken setToken;

  String get baseUrl => baseUrlCallback == null ? '' : baseUrlCallback();
  String get basicAuth => basicAuthCallback == null ? '' : basicAuthCallback();
  String get token => jwtTokenCallback == null ? '' : jwtTokenCallback();

  static Future<bool> init(WebClientStringCallback baseUrlCallback,
      WebClientStringCallback basicAuthCallback,
      {WebClientStringCallback jwtTokenCallback}) async {
    if (_this == null)
      _this = WebClient._(baseUrlCallback, basicAuthCallback, jwtTokenCallback);
    return Future.value(true);
  }

  static WebClient _this;
  WebClient._(
      this.baseUrlCallback, this.basicAuthCallback, this.jwtTokenCallback)
      : super();

  static WebClient get current => _this;
  //static const baseUrl = 'http://104.248.147.117:5011';
  //static const basicAuth = 'Basic Y29kZWxhYjpjMGQzbEBiUEAkJA==';

  String queryString(String api,
      {List<dynamic> params, bool basicAuthInQuery = false}) {
    return '$baseUrl$api' +
        (params == null
            ? ''
            : '/${params.map((param) => '$param').join('/').toString()}') +
        (basicAuthInQuery ? '?BasicAuthorization=$basicAuth' : '');
  }

  String appendBasicAuth(String url){
    return url + '?BasicAuthorization=$basicAuth';
  }

  Future<WebClientResult> get(String api,
      {List<dynamic> params, bool basicAuthInQuery = false}) async {
    final String url = queryString(api, params: params, basicAuthInQuery: basicAuthInQuery);

    final http.Response response = await http.get(url, headers: {
      'BasicAuthorization': 'Basic $basicAuth',
      'Authorization': 'Bearer $token'
    });

    return WebClientResult.fromHttpResponse(response);
  }

  Future<WebClientResult> post(String api, Map data) async {
    final String url = '$baseUrl$api';
    final http.Response response = await http.post(
      url,
      body: json.encode(data),
      headers: {
        'Accept': 'application/json',
        'Content-Type': 'application/json',
        'BasicAuthorization': 'Basic $basicAuth',
        'Authorization': 'Bearer $token'
      },
    );
    return WebClientResult.fromHttpResponse(response);
  }

  Future<WebClientResult> put(String api, Map data) async {
    final String url = '$baseUrl$api';
    final http.Response response = await http.put(
      url,
      body: json.encode(data),
      headers: {
        'Accept': 'application/json',
        'Content-Type': 'application/json',
        'BasicAuthorization': 'Basic $basicAuth',
        'Authorization': 'Bearer $token'
      },
    );

    return WebClientResult.fromHttpResponse(response);
  }
}

class WebClientResult {
  final int statusCode;
  final bool ok;
  final dynamic data;
  final String error;
  //final bool isTypeOfException;

  WebClientResult({
    this.statusCode = -1,
    this.ok,
    this.error,
    this.data,
  });

  static WebClientResult success({dynamic value}) {
    return WebClientResult(statusCode: 200, ok: true, data: value);
  }

  static WebClientResult failed(String error, {int statusCode = -1}) {
    return WebClientResult(ok: false, error: error);
  }

  static WebClientResult fromHttpResponse(http.Response response) {
    if (response.statusCode == 200) {
      if (response.body?.isEmpty ?? true) {
        return success();
      }
      try {
        final jsonResponse = json.decode(response.body);
        return success(value: jsonResponse);
      } on Exception catch (e) {
        return failed(e.toString());
      }
    }
    return failed(response.body, statusCode: response.statusCode);
  }

  WebClientResult copy(dynamic data) {
    return WebClientResult(
        statusCode: statusCode, ok: ok, error: error, data: data);
  }
}
